/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package business.collections;

import business.base.BaseSubject;
import business.base.IList;
import business.base.IListItem;
import business.entities.Result;
import java.util.HashMap;
import java.util.List;

/**
 *
 * @author balikm1
 */
public class ResultList implements IList {

    private final String INSERT_STMT = "INSERT INTO APP.RESULTS (Student_ID, Subject_ID, Exam, Credit, Grade) VALUES (?, ?, ?, ?, ?)";
    private final String UPDATE_STMT = "UPDATE APP.RESULTS SET Student_ID = ?, Subject_ID = ?, Exam = ?, Credit = ?, Grade = ? WHERE ID = ?";
    private final String DELETE_STMT = "DELETE FROM APP.RESULTS WHERE ID = ?";
    private final String SELECT_CURRENT_STMT = "SELECT ID, Student_ID, Subject_ID, Exam, Credit, Grade FROM APP.RESULTS WHERE Student_ID = ? AND ID = ?";
    private final String SELECT_NEXT_STMT = "SELECT ID, Student_ID, Subject_ID, Exam, Credit, Grade FROM APP.RESULTS WHERE Student_ID = ? AND ID > ? ORDER BY ID";
    private final String SELECT_PREVIOUS_STMT = "SELECT ID, Student_ID, Subject_ID, Exam, Credit, Grade FROM APP.RESULTS WHERE Student_ID = ? AND ID < ? ORDER BY ID DESC";
    private final String SELECT_NEXT_COUNT_STMT = "SELECT COUNT(*) FROM APP.RESULTS WHERE Student_ID = ? AND ID > ?";
    private final String SELECT_PREVIOUS_COUNT_STMT = "SELECT COUNT(*) FROM APP.RESULTS WHERE Student_ID = ? AND ID < ?";
    
    private int id = 0;
    private int studentId = 0;

    public ResultList(int studentId)
    {
        this.studentId = studentId;
    }

    public int getStudentId()
    {
        return studentId;
    }

    public void saveOrUpdate(Result result)
    {
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, studentId);
        params.put(2, result.getSubject().getId());
        params.put(3, result.hasExamResult() ? 1 : 0);
        params.put(4, result.hasCredit() ? 1 : 0);
        if (result.hasExamResult())
            try {
                params.put(5, result.getExamResult());
            } catch (Exception e) {
                params.put(5, 0);
            }
        else
            params.put(5, 0);

        if (result.getId() == 0)
        {
            result.setId(SQLManager.getInstance().executeAutoincrementInsert(INSERT_STMT, params));
        } else {
            params.put(6, result.getId());
            SQLManager.getInstance().executeUpdate(UPDATE_STMT, params);
        }
    }

    public void remove(Result result)
    {
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, result.getId());

        SQLManager.getInstance().executeUpdate(DELETE_STMT, params);
    }

    // <editor-fold defaultstate="collapsed" desc="private methods">

    private Result getById(int id, String query)
    {
        Result s = null;

        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, studentId);
        params.put(2, id);

        List<Object> result = SQLManager.getInstance().executeQuery(query, params);
        if (result.size() == 6)
        {
            BaseSubject subject = SubjectList.getInstance().findById(((Integer)result.get(2)).intValue());
            s = new Result(subject);
            if (result.get(4) != null && ((Integer)result.get(4)).intValue() == 1)
                try {
                    s.setCredit();
                } catch(Exception e) {}
            if (result.get(3) != null && ((Integer)result.get(3)).intValue() == 1)
                try {
                    s.setExamResult(((Integer)result.get(5)).intValue());
                } catch(Exception e) {}
            s.setId(((Integer)result.get(0)).intValue());
        }

        return s;
    }

    private int getCount(int id, String query)
    {
        int count = 0;
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, studentId);
        params.put(2, id);
        List<Object> result = SQLManager.getInstance().executeQuery(query, params);
        count = (Integer)result.get(0);
        return count;
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="IList methods">

    public IListItem current() {
        IListItem s;
        if ((s = getById(this.id, SELECT_CURRENT_STMT)) == null)
            s = next();
        return s;
    }

    public IListItem next() {
        Result s = getById(id, SELECT_NEXT_STMT);
        if (s != null)
            id = s.getId();
        return s;
    }

    public boolean hasNext() {
        return getCount(id, SELECT_NEXT_COUNT_STMT) > 0;
    }

    public IListItem previous() {
        Result s = getById(id, SELECT_PREVIOUS_STMT);
        if (s != null)
            id = s.getId();
        return s;
    }

    public boolean hasPrevious() {
        return getCount(id, SELECT_PREVIOUS_COUNT_STMT) > 0;
    }

    // </editor-fold>

}
