/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package business.collections;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author balikm1
 */
public class SQLManager {
    
    private final String JDBC_DRIVER = "org.apache.derby.jdbc.EmbeddedDriver";
    private final String CONNECTION_STRING = "jdbc:derby:JavaKOSdb;create=true";
    private final String USER = "user";
    private final String PASSWORD = "password";
    
    private static SQLManager instance;

    private SQLManager()
    {
        try
        {
            // JDBC driver
            Class.forName( JDBC_DRIVER );
        } catch (java.lang.ClassNotFoundException e) {
            System.err.print("ClassNotFoundException: ");
            System.err.println(e.getMessage());
        }
    }

    public static SQLManager getInstance()
    {
        if (instance == null)
            instance = new SQLManager();
        return instance;
    }

    /**
     * Metoda používaná pro příkaz SELECT. Počítá pouze s jedním vráceným záznamem. Pro čtení více záznamů by bylo potřeba metodu rozšířit.
     * @param sql SQL dotaz s možnými parametry
     * @param params Parametry dosazovane do SQL PreparedStatement ve tvaru "poradi - hodnota"
     * @return Kolekce hodnot sloupců prvního vráceného záznamu.
     */
    public List<Object> executeQuery(String sql, HashMap<Integer,Object> params) {
        List<Object> result = new ArrayList<Object>();
        PreparedStatement stmt = null;
        try
        {
            // vytvoreni spojeni s databazi
            Connection conn = DriverManager.getConnection(CONNECTION_STRING, USER, PASSWORD);

            try {
                stmt = conn.prepareStatement(sql);
                for(Map.Entry<Integer, Object> e : params.entrySet())
                    stmt.setObject(e.getKey(), e.getValue());

                stmt.setMaxRows(1);
                ResultSet rs = stmt.executeQuery();

                if (rs.next()) {
                    ResultSetMetaData rsMetaData = rs.getMetaData();
                    int columnCount = rsMetaData.getColumnCount();
                    for (int i=1; i<=columnCount; i++)
                        result.add(rs.getObject(i));
                }

                rs.close();
                stmt.close();
            } finally {
                conn.close();
            }
        } catch (SQLException ex) {
            System.err.println("SQLException: " + ex.getMessage());
        }
        return result;
    }

    /**
     * Metoda používaná pro SQL příkazy pro změnu dat
     * @param sql SQL dotaz s možnými parametry
     * @param params Parametry dosazovane do SQL PreparedStatement ve tvaru "poradi - hodnota"
     */
    public void executeUpdate(String sql, HashMap<Integer,Object> params) {
        Connection conn = null;
        PreparedStatement stmt = null;
        try
        {
            // vytvoreni spojeni s databazi
            conn = DriverManager.getConnection(CONNECTION_STRING, USER, PASSWORD);

            try {
                stmt = conn.prepareStatement(sql);
                for(Map.Entry<Integer, Object> e : params.entrySet())
                    stmt.setObject(e.getKey(), e.getValue());

                stmt.executeUpdate();

                stmt.close();
            } finally {
                conn.close();
            }
        } catch (SQLException ex) {
            System.err.println("SQLException: " + ex.getMessage());
        }
    }

    /**
     * Metoda používaná pro příkaz INSERT, který využívá autoinkrementace primárního klíče
     * @param sql SQL dotaz s možnými parametry
     * @param params Parametry dosazovane do SQL PreparedStatement ve tvaru "poradi - hodnota"
     * @return Primarni klic vytvoreneho zaznamu
     */
    public int executeAutoincrementInsert(String sql, HashMap<Integer,Object> params) {
        Connection conn = null;
        PreparedStatement stmt = null;
        int key = 0;
        try
        {
            // vytvoreni spojeni s databazi
            conn = DriverManager.getConnection(CONNECTION_STRING, USER, PASSWORD);

            try {
                stmt = conn.prepareStatement(sql);
                for(Map.Entry<Integer, Object> e : params.entrySet())
                    stmt.setObject(e.getKey(), e.getValue());

                stmt.executeUpdate();
                ResultSet keys = stmt.getGeneratedKeys();

                keys.next();
                key = keys.getInt(1);

                keys.close();
                stmt.close();
            } finally {
                conn.close();
            }
        } catch (SQLException ex) {
            System.err.println("SQLException: " + ex.getMessage());
        }
        return key;
    }

}
