/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package business.collections;

import business.base.IList;
import business.base.IListItem;
import business.entities.Student;
import java.util.HashMap;
import java.util.List;

/**
 *
 * @author balikm1
 */
public class StudentList implements IList {

    private final String INSERT_STMT = "INSERT INTO APP.STUDENT (FirstName, LastName, DateOfBirth, PersonalNumber) VALUES (?, ?, ?, ?)";
    private final String UPDATE_STMT = "UPDATE APP.STUDENT SET FirstName = ?, LastName = ?, DateOfBirth = ?, PersonalNumber = ? WHERE ID = ?";
    private final String DELETE_STMT = "DELETE FROM APP.STUDENT WHERE ID = ?";
    private final String SELECT_CURRENT_STMT = "SELECT ID, FirstName, LastName, DateOfBirth, PersonalNumber FROM APP.STUDENT WHERE ID = ?";
    private final String SELECT_NEXT_STMT = "SELECT ID, FirstName, LastName, DateOfBirth, PersonalNumber FROM APP.STUDENT WHERE ID > ? ORDER BY ID";
    private final String SELECT_PREVIOUS_STMT = "SELECT ID, FirstName, LastName, DateOfBirth, PersonalNumber FROM APP.STUDENT WHERE ID < ? ORDER BY ID DESC";
    private final String SELECT_NEXT_COUNT_STMT = "SELECT COUNT(*) FROM APP.STUDENT WHERE ID > ?";
    private final String SELECT_PREVIOUS_COUNT_STMT = "SELECT COUNT(*) FROM APP.STUDENT WHERE ID < ?";

    private int id = 0;
    private static StudentList instance;

    private StudentList() { }

    public static StudentList getInstance()
    {
        if (instance == null)
        {
            instance = new StudentList();
        }
        return instance;
    }

    public void saveOrUpdate(Student student)
    {
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, student.getFirstName());
        params.put(2, student.getLastName());
        params.put(3, new java.sql.Date(student.getDateOfBirth().getTime()));
        params.put(4, student.getPersonalNumber());

        if (student.getId() == 0)
        {
            student.setId(SQLManager.getInstance().executeAutoincrementInsert(INSERT_STMT, params));
        } else {
            params.put(5, student.getId());
            SQLManager.getInstance().executeUpdate(UPDATE_STMT, params);
        }
    }

    public void remove(Student student)
    {
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, student.getId());

        SQLManager.getInstance().executeUpdate(DELETE_STMT, params);
    }

    // <editor-fold defaultstate="collapsed" desc="private methods">

    private Student getById(int id, String query)
    {
        Student s = null;

        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, id);

        List<Object> result = SQLManager.getInstance().executeQuery(query, params);
        if (result.size() == 5)
        {
            s = new Student((String)result.get(1), (String)result.get(2), new java.util.Date(((java.sql.Date)result.get(3)).getTime()), ((Integer)result.get(4)).intValue());
            s.setId(((Integer)result.get(0)).intValue());
        }

        return s;
    }

    private int getCount(int id, String query)
    {
        int count = 0;
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, id);
        List<Object> result = SQLManager.getInstance().executeQuery(query, params);
        count = (Integer)result.get(0);
        return count;
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="IList methods">

    public IListItem current() {
        IListItem s;
        if ((s = getById(this.id, SELECT_CURRENT_STMT)) == null)
            s = next();
        return s;
    }

    public IListItem next() {
        Student s = getById(id, SELECT_NEXT_STMT);
        if (s != null)
            id = s.getId();
        return s;
    }

    public boolean hasNext() {
        return getCount(id, SELECT_NEXT_COUNT_STMT) > 0;
    }

    public IListItem previous() {
        Student s = getById(id, SELECT_PREVIOUS_STMT);
        if (s != null)
            id = s.getId();
        return s;
    }

    public boolean hasPrevious() {
        return getCount(id, SELECT_PREVIOUS_COUNT_STMT) > 0;
    }

    // </editor-fold>

}
