/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package business.collections;

import business.base.BaseSubject;
import business.base.IHasCredit;
import business.base.IHasExam;
import business.base.IList;
import business.base.IListItem;
import business.entities.LabSubject;
import business.entities.RegularSubject;
import business.entities.TheoreticalSubject;
import java.util.HashMap;
import java.util.List;

/**
 *
 * @author balikm1
 */
public class SubjectList implements IList {

    private final String INSERT_STMT = "INSERT INTO APP.SUBJECT (SubjectName, Code, Credits, HasCredit, HasExam) VALUES (?, ?, ?, ?, ?)";
    private final String UPDATE_STMT = "UPDATE APP.SUBJECT SET SubjectName = ?, Code = ?, Credits = ?, HasCredit = ?, HasExam = ? WHERE ID = ?";
    private final String DELETE_STMT = "DELETE FROM APP.SUBJECT WHERE ID = ?";
    private final String SELECT_CURRENT_STMT = "SELECT ID, SubjectName, Code, Credits, HasCredit, HasExam FROM APP.SUBJECT WHERE ID = ?";
    private final String SELECT_NEXT_STMT = "SELECT ID, SubjectName, Code, Credits, HasCredit, HasExam FROM APP.SUBJECT WHERE ID > ? ORDER BY ID";
    private final String SELECT_PREVIOUS_STMT = "SELECT ID, SubjectName, Code, Credits, HasCredit, HasExam FROM APP.SUBJECT WHERE ID < ? ORDER BY ID DESC";
    private final String SELECT_NEXT_COUNT_STMT = "SELECT COUNT(*) FROM APP.SUBJECT WHERE ID > ?";
    private final String SELECT_PREVIOUS_COUNT_STMT = "SELECT COUNT(*) FROM APP.SUBJECT WHERE ID < ?";

    private int id = 0;
    private static SubjectList instance;

    private SubjectList() { }

    public static SubjectList getInstance()
    {
        if (instance == null)
        {
            instance = new SubjectList();
        }
        return instance;
    }

    public void saveOrUpdate(BaseSubject subject)
    {
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, subject.getName());
        params.put(2, subject.getCode());
        params.put(3, subject.getCredits());
        params.put(4, (subject instanceof IHasCredit) ? "Y" : "N");
        params.put(5, (subject instanceof IHasExam) ? "Y" : "N");

        if (subject.getId() == 0)
        {
            subject.setId(SQLManager.getInstance().executeAutoincrementInsert(INSERT_STMT, params));
        } else {
            params.put(6, subject.getId());
            SQLManager.getInstance().executeUpdate(UPDATE_STMT, params);
        }
    }

    public void remove(BaseSubject subject)
    {
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, subject.getId());

        SQLManager.getInstance().executeUpdate(DELETE_STMT, params);
    }

    public BaseSubject findById(int id)
    {
        return getById(id, SELECT_CURRENT_STMT);
    }

    // <editor-fold defaultstate="collapsed" desc="private methods">

    private BaseSubject getById(int id, String query)
    {
        BaseSubject s = null;

        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, id);

        List<Object> result = SQLManager.getInstance().executeQuery(query, params);
        if (result.size() == 6)
        {
            if (result.get(4).equals("Y") && result.get(5).equals("Y"))
                s = new RegularSubject((String)result.get(1), (String)result.get(2), ((Integer)result.get(3)).intValue());
            else if(result.get(4).equals("Y") && result.get(5).equals("N"))
                s = new LabSubject((String)result.get(1), (String)result.get(2), ((Integer)result.get(3)).intValue());
            else if(result.get(4).equals("N") && result.get(5).equals("Y"))
                s = new TheoreticalSubject((String)result.get(1), (String)result.get(2), ((Integer)result.get(3)).intValue());
            else
                return null;
            s.setId(((Integer)result.get(0)).intValue());
        }

        return s;
    }

    private int getCount(int id, String query)
    {
        int count = 0;
        HashMap<Integer, Object> params = new HashMap<Integer, Object>();
        params.put(1, id);
        List<Object> result = SQLManager.getInstance().executeQuery(query, params);
        count = (Integer)result.get(0);
        return count;
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="IList methods">

    public IListItem current() {
        IListItem s;
        if ((s = getById(this.id, SELECT_CURRENT_STMT)) == null)
            s = next();
        return s;
    }

    public IListItem next() {
        BaseSubject s = getById(id, SELECT_NEXT_STMT);
        if (s != null)
            id = s.getId();
        return s;
    }

    public boolean hasNext() {
        return getCount(id, SELECT_NEXT_COUNT_STMT) > 0;
    }

    public IListItem previous() {
        BaseSubject s = getById(id, SELECT_PREVIOUS_STMT);
        if (s != null)
            id = s.getId();
        return s;
    }

    public boolean hasPrevious() {
        return getCount(id, SELECT_PREVIOUS_COUNT_STMT) > 0;
    }

    // </editor-fold>

}
