/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package business.entities;

import business.base.BaseSubject;
import business.base.DBEntity;
import business.base.IList;
import business.base.IListItem;
import business.collections.ResultList;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;

/**
 *
 * @author balikm1
 */
public class Student extends DBEntity implements IListItem {

    private String firstName;
    private String lastName;
    private Date dateOfBirth;
    private final int personalNumber;

    public Student(String firstName, String lastName, Date dateOfBirth, int personalNumber)
    {
        this.firstName = firstName;
        this.lastName = lastName;
        this.dateOfBirth = dateOfBirth;
        this.personalNumber = personalNumber;
    }
    
    // <editor-fold defaultstate="collapsed" desc="Getters & Setters">

    public Date getDateOfBirth() {
        return dateOfBirth;
    }

    public void setDateOfBirth(Date dateOfBirth) {
        this.dateOfBirth = dateOfBirth;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public int getPersonalNumber() {
        return personalNumber;
    }

    public ResultList getResults() {
        ResultList results = new ResultList(this.getId());
        return results;
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="Object overrides">

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Student other = (Student) obj;
        if (this.personalNumber != other.personalNumber) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 79 * hash + this.personalNumber;
        return hash;
    }

    @Override
    public String toString() {
        return "Student{" + "firstName=" + firstName + "lastName=" + lastName + "dateOfBirth=" + new SimpleDateFormat("yyyy-MM-dd").format(dateOfBirth) + "personalNumber=" + personalNumber + '}';
    }

    // </editor-fold>

    // <editor-fold desc="Data manipulation methods">

    public void registerSubject(BaseSubject subject)
    {
        if (this.getId() > 0) {
            ResultList results = new ResultList(this.getId());
            results.saveOrUpdate(new Result(subject));
        }
    }

    public void cancelSubject(BaseSubject subject)
    {
        ResultList results = new ResultList(this.getId());
        results.remove(new Result(subject));
    }

    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="IListItem methods">

    public HashMap<String, String> getPublicAttributes() {
        HashMap<String, String> map = new LinkedHashMap<String, String>();
        map.put("Jméno", this.firstName);
        map.put("Příjmení", this.lastName);
        map.put("Datum narození", this.dateOfBirth != null ? new SimpleDateFormat("yyyy-MM-dd").format(this.dateOfBirth) : "");
        map.put("Osobní číslo", Integer.toString(this.personalNumber));
        return map;
    }

    public boolean hasSubitems() {
        return true;
    }

    public IList getSubitems() {
        ResultList results = new ResultList(this.getId());
        return results;
    }

    public boolean isEditable() {
        return true;
    }

    public boolean canCreate() {
        return true;
    }

    // </editor-fold>

}
